//
//  AppDelegate.m
//  SqliteJoinQuery
//
//  Created by Merlyn on 12/4/13.
//  Copyright (c) 2013 The App Lady. All rights reserved.
//

#import "AppDelegate.h"

@implementation AppDelegate

- (NSString *)docPath {
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *docsFolder = [paths objectAtIndex:0];
	return [docsFolder stringByAppendingPathComponent:@"vintageDatabase.sqlite"];
}

- (void) openDatabase {
	// Get full path to the database file
	NSString *dbFile = [self docPath];
	
	// Check to see if the database file exists in the Documents folder
	BOOL dbExist = [[NSFileManager defaultManager] fileExistsAtPath:dbFile];
	if(!dbExist) {
		// It doesn't, so create and open the vintageDatabase file in the Documents folder
		int result = sqlite3_open([dbFile UTF8String], &dbHandle);
		
		if (result != SQLITE_OK) {
			// Unable to open the database file, so close it
			[self closeDatabase];
			
			// Display the error message the sqlite3_open_vs() function returned
			NSLog(@"openDatabase Error:\n%s",sqlite3_errmsg(dbHandle));
		}
	}
	sqlite3_open([dbFile UTF8String], &dbHandle);
}

- (void)closeDatabase {
	// Close the SQLite db file
	int result = sqlite3_close(dbHandle);
	
	if (result != SQLITE_OK) {
		// Unable to close the db file, so display the error message the sqlite3_close() function returned
		NSLog(@"closeDatabase Error:\n'%s'", sqlite3_errmsg(dbHandle));
	}
}

- (NSMutableArray *)fetchAllVideos {
	// Create an initialize array objects
	NSMutableArray *dbRecords = [[NSMutableArray alloc] init];
	NSArray *currentRow = [[NSArray alloc] init];
	
	const char *sql = "SELECT rowId, videoName, rentalPrice, category FROM videos";
	sqlite3_stmt *compiledStatement;
	
	//1. Prepare the sql statement
	int result = sqlite3_prepare_v2(dbHandle, sql, -1, &compiledStatement, NULL);
	
	if(result != SQLITE_OK) {
		NSLog(@"FetchAllVideos method error:\n%s", sqlite3_errmsg(dbHandle));
	} else {
		//2. Execute the prepared statement
		while(sqlite3_step(compiledStatement) == SQLITE_ROW) {
			// Convert each record's column value to an object
			NSNumber *dbRowid = [NSNumber numberWithInt:sqlite3_column_int(compiledStatement, 0)];
			
			NSString *dbVideoName = [NSString stringWithUTF8String:
											 (char *)sqlite3_column_text(compiledStatement, 1)];
			
			NSDecimalNumber *dbRentalPrice = [NSDecimalNumber decimalNumberWithMantissa:
														 sqlite3_column_double(compiledStatement, 2) exponent:-2 isNegative:NO];
			
			NSString *dbCategory = [NSString stringWithUTF8String:
											(char *)sqlite3_column_text(compiledStatement, 3)];
			
			// Place converted record's columns in the currentRow object
			currentRow = @[dbRowid,dbVideoName,dbRentalPrice,dbCategory];
			
			// Add the currentRow object in the dbRecords object
			[dbRecords addObject:currentRow];
		}
	}
	
	//3. Finalize the prepared statement
	sqlite3_finalize(compiledStatement);
	
	// Return records fetch from the videos table
	return dbRecords;
}

- (NSMutableArray *)fetchPhotosByAlbum:(int)albumId {
	// Create and initialize a mutable array
	NSMutableArray *photos = [[NSMutableArray alloc] init];
	
	const char *sql = "SELECT photoUrl FROM photos WHERE aid=?";
	sqlite3_stmt *compiledStatement;
	
	//1. Prepare the sql statement
	int result = sqlite3_prepare_v2(dbHandle, sql, -1, &compiledStatement, NULL);
	
	// Bind the method's argument
	sqlite3_bind_int(compiledStatement, 1, albumId);
	
	if(result != SQLITE_OK) {
		NSLog(@"fetchPhotosByAlbum method error:\n%s", sqlite3_errmsg(dbHandle));
	} else {
		//2. Execute the prepared statement
		while(sqlite3_step(compiledStatement) == SQLITE_ROW) {
			NSString *dbPhotoUrl = [NSString stringWithUTF8String:(char *)sqlite3_column_text(compiledStatement, 0)];
			
			// Place rows the SELECT statement returned in the mutable array
			[photos addObject:dbPhotoUrl];
		}
	}
	
	//3. Finalize the prepared statement
	sqlite3_finalize(compiledStatement);
	
	// Return an empty mutable array or rows the SELECT statement returned
	return photos;
}

- (NSMutableArray *)fetchAlbums {
	// Create an initialize array object
	NSMutableArray *dbRecords = [[NSMutableArray alloc] init];
	
	const char *sql = "SELECT albumName FROM albums ORDER BY albumName";
	sqlite3_stmt *compiledStatement;
	
	//1. Prepare the sql statement
	int result = sqlite3_prepare_v2(dbHandle, sql, -1, &compiledStatement, NULL);
	
	if(result != SQLITE_OK) {
		NSLog(@"fetchAlbums method error:\n%s", sqlite3_errmsg(dbHandle));
	} else {
		//2. Execute the prepared statement
		while(sqlite3_step(compiledStatement) == SQLITE_ROW) {
			NSString *albumName = [NSString stringWithUTF8String:
										  (char *)sqlite3_column_text(compiledStatement, 0)];
			
			// Add the db row in the mutable array
			[dbRecords addObject:albumName];
		}
	}
	
	//3. Finalize the prepared statement
	sqlite3_finalize(compiledStatement);
	
	// Return the db rows
	return dbRecords;
}

- (NSMutableArray *)joinQuery {
	NSMutableArray *dbRows = [[NSMutableArray alloc] init];
	int totalRows = 0;
	
	// This query returns the total number of links the album contain
	const char *queryOne = "SELECT count(photos.pid) AS totalRows FROM albums INNER JOIN photos ON albums.aid=photos.aid WHERE albums.albumName = \"Fathers Day\"";
	sqlite3_stmt *compiledStatement1;
	
	// This query returns all photo urls there are in the album
	const char *queryTwo = "SELECT photos.photoUrl FROM albums INNER JOIN photos ON albums.aid=photos.aid WHERE albums.albumName = \"Fathers Day\"";
	sqlite3_stmt *compiledStatement2;
	
	// Prepare both statements and display errors, if any
	int resultOne = sqlite3_prepare_v2(dbHandle, queryOne, -1, &compiledStatement1, NULL);
	int resultTwo = sqlite3_prepare_v2(dbHandle, queryTwo, -1, &compiledStatement2, NULL);
	
	if(resultOne != SQLITE_OK) {
		NSLog(@"joinQuery error:\n%s", sqlite3_errmsg(dbHandle));
	} else if (resultTwo != SQLITE_OK){
		NSLog(@"joinQuery error:\n%s", sqlite3_errmsg(dbHandle));
	}
	
	// Both queries returned no erros, so execute the queryOne
	resultOne = sqlite3_step(compiledStatement1);
	
	if (resultOne == SQLITE_ROW) {
		// Place the single column queryOne returned in a variable
		totalRows = sqlite3_column_int(compiledStatement1, 0);
	}
	
	// Validate the totalRows variable
	if (totalRows > 0) {
		// Execute queryTwo in a while loop
		while(sqlite3_step(compiledStatement2) == SQLITE_ROW) {
			// Convert queryTwo's column values from C data types to objective-C data types,
			// before assinging them to Objective-C object variable
			NSString *photoUrls = [NSString stringWithUTF8String:
										  (char *)sqlite3_column_text(compiledStatement2, 0)];
			
			// Add each database row in the mutable array
			[dbRows addObject:photoUrls];
		}
	}
	// Finalize both prepared statements
	sqlite3_finalize(compiledStatement1);
	sqlite3_finalize(compiledStatement2);
	
	/***************** DEBUG WINDOW OUTPUT *******************/
	for(int i=0; i < [dbRows count]; i++) {
		NSLog(@"%i. %@", i+1, [dbRows objectAtIndex:i]);
	}
	/*********************************************************/
	// return an empty array or rows returned from queryTwo
	return dbRows;
}

- (NSArray *)fetchVideoByTitle:(NSString *)title
{
	// Create and initialize a mutable array
	NSArray *dbRecord = [[NSArray alloc] init];
	
	const char *sql = "SELECT rowId, videoName, rentalPrice, category FROM videos WHERE videoName=?";
	sqlite3_stmt *compiledStatement;
	
	//1. Prepare the sql statement
	int result = sqlite3_prepare_v2(dbHandle, sql, -1, &compiledStatement, NULL);
	
	// Bind the method's argument
	sqlite3_bind_text(compiledStatement, 1, [title UTF8String], -1, NULL);
	
	if(result != SQLITE_OK) {
		NSLog(@"fetchVideoByTitle method error:\n%s", sqlite3_errmsg(dbHandle));
	} else {
		//2. Execute the prepared statement
		result = sqlite3_step(compiledStatement) == SQLITE_ROW;
		if(result) {
			// Convert the record's columns to Objective-C objects before assigning them to object variables
			NSNumber *rowId = [NSNumber numberWithInt:sqlite3_column_int(compiledStatement, 0)];
			
			NSString *videoName = [NSString stringWithUTF8String:(const char *)sqlite3_column_text(compiledStatement, 1)];
			
			NSString *rentalPrice = [NSString stringWithUTF8String:(const char *)sqlite3_column_text(compiledStatement, 2)];
			
			NSString *category = [NSString stringWithUTF8String:(const char *)sqlite3_column_text(compiledStatement, 3)];
			
			// Place above database record's fields in the array
			dbRecord = @[rowId, videoName, rentalPrice, category];
		} else {
			NSLog(@"fetchVideoByTitle method error:\n%s",sqlite3_errmsg(dbHandle));
		}
	}
	
	//3. Finalize the prepared statement
	sqlite3_finalize(compiledStatement);
	
	// Return an empty array or the record the SELECT statement returned
	return dbRecord;
}

/**********************************************************************************/

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions
{
    // Override point for customization after application launch.
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application
{
	// Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
	// Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application
{
	// Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later. 
	// If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application
{
	// Called as part of the transition from the background to the inactive state; here you can undo many of the changes made on entering the background.
}

- (void)applicationDidBecomeActive:(UIApplication *)application
{
	// Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
}

- (void)applicationWillTerminate:(UIApplication *)application
{
	// Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
}

@end
